"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import Nullable, OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Activities(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.ActivitiesFilter, models.ActivitiesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.ActivitiesSort, models.ActivitiesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmActivitiesAllResponse]:
        r"""List activities

        List activities

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.ActivitiesFilter]
            ),
            sort=utils.get_pydantic_model(sort, Optional[models.ActivitiesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/activities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmActivitiesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesAllResponse(
                get_activities_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetActivitiesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.ActivitiesFilter, models.ActivitiesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.ActivitiesSort, models.ActivitiesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmActivitiesAllResponse]:
        r"""List activities

        List activities

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.ActivitiesFilter]
            ),
            sort=utils.get_pydantic_model(sort, Optional[models.ActivitiesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/activities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmActivitiesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesAllResponse(
                get_activities_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetActivitiesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def create(
        self,
        *,
        type_: Nullable[models.ActivityType],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        activity_datetime: OptionalNullable[str] = UNSET,
        duration_seconds: OptionalNullable[int] = UNSET,
        user_id: OptionalNullable[str] = UNSET,
        account_id: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        opportunity_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        campaign_id: OptionalNullable[str] = UNSET,
        case_id: OptionalNullable[str] = UNSET,
        asset_id: OptionalNullable[str] = UNSET,
        contract_id: OptionalNullable[str] = UNSET,
        product_id: OptionalNullable[str] = UNSET,
        solution_id: OptionalNullable[str] = UNSET,
        custom_object_id: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        note: OptionalNullable[str] = UNSET,
        location: OptionalNullable[str] = UNSET,
        location_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        all_day_event: OptionalNullable[bool] = UNSET,
        private: OptionalNullable[bool] = UNSET,
        group_event: OptionalNullable[bool] = UNSET,
        event_sub_type: OptionalNullable[str] = UNSET,
        group_event_type: OptionalNullable[str] = UNSET,
        child: OptionalNullable[bool] = UNSET,
        archived: OptionalNullable[bool] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        show_as: OptionalNullable[models.ShowAs] = UNSET,
        done: OptionalNullable[bool] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        activity_date: OptionalNullable[str] = UNSET,
        end_date: OptionalNullable[str] = UNSET,
        recurrent: Optional[bool] = None,
        reminder_datetime: OptionalNullable[str] = UNSET,
        reminder_set: OptionalNullable[bool] = UNSET,
        video_conference_url: OptionalNullable[str] = UNSET,
        video_conference_id: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        attendees: Optional[
            Union[
                List[models.ActivityAttendeeInput],
                List[models.ActivityAttendeeInputTypedDict],
            ]
        ] = None,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesAddResponse:
        r"""Create activity

        Create activity

        :param type: The type of the activity
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param activity_datetime: The date and time of the activity
        :param duration_seconds: The duration of the activity in seconds
        :param user_id: The user related to the activity
        :param account_id: The account related to the activity
        :param contact_id: The contact related to the activity
        :param company_id: The company related to the activity
        :param opportunity_id: The opportunity related to the activity
        :param lead_id: The lead related to the activity
        :param owner_id: The owner of the activity
        :param campaign_id: The campaign related to the activity
        :param case_id: The case related to the activity
        :param asset_id: The asset related to the activity
        :param contract_id: The contract related to the activity
        :param product_id: The product related to the activity
        :param solution_id: The solution related to the activity
        :param custom_object_id: The custom object related to the activity
        :param title: The title of the activity
        :param description: A description of the activity
        :param note: An internal note about the activity
        :param location: The location of the activity
        :param location_address:
        :param all_day_event: Whether the Activity is an all day event or not
        :param private: Whether the Activity is private or not
        :param group_event: Whether the Activity is a group event or not
        :param event_sub_type: The sub type of the group event
        :param group_event_type: The type of the group event
        :param child: Whether the activity is a child of another activity or not
        :param archived: Whether the activity is archived or not
        :param deleted: Whether the activity is deleted or not
        :param show_as:
        :param done: Whether the Activity is done or not
        :param start_datetime: The start date and time of the activity
        :param end_datetime: The end date and time of the activity
        :param activity_date: The date of the activity
        :param end_date: The end date of the activity
        :param recurrent: Whether the activity is recurrent or not
        :param reminder_datetime: The date and time of the reminder
        :param reminder_set: Whether the reminder is set or not
        :param video_conference_url: The URL of the video conference
        :param video_conference_id: The ID of the video conference
        :param custom_fields: Custom fields of the activity
        :param attendees:
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            activity=models.ActivityInput(
                activity_datetime=activity_datetime,
                duration_seconds=duration_seconds,
                user_id=user_id,
                account_id=account_id,
                contact_id=contact_id,
                company_id=company_id,
                opportunity_id=opportunity_id,
                lead_id=lead_id,
                owner_id=owner_id,
                campaign_id=campaign_id,
                case_id=case_id,
                asset_id=asset_id,
                contract_id=contract_id,
                product_id=product_id,
                solution_id=solution_id,
                custom_object_id=custom_object_id,
                type=type_,
                title=title,
                description=description,
                note=note,
                location=location,
                location_address=utils.get_pydantic_model(
                    location_address, Optional[models.Address]
                ),
                all_day_event=all_day_event,
                private=private,
                group_event=group_event,
                event_sub_type=event_sub_type,
                group_event_type=group_event_type,
                child=child,
                archived=archived,
                deleted=deleted,
                show_as=show_as,
                done=done,
                start_datetime=start_datetime,
                end_datetime=end_datetime,
                activity_date=activity_date,
                end_date=end_date,
                recurrent=recurrent,
                reminder_datetime=reminder_datetime,
                reminder_set=reminder_set,
                video_conference_url=video_conference_url,
                video_conference_id=video_conference_id,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                attendees=utils.get_pydantic_model(
                    attendees, Optional[List[models.ActivityAttendeeInput]]
                ),
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/crm/activities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.activity, False, False, "json", models.ActivityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmActivitiesAddResponse(
                create_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        type_: Nullable[models.ActivityType],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        activity_datetime: OptionalNullable[str] = UNSET,
        duration_seconds: OptionalNullable[int] = UNSET,
        user_id: OptionalNullable[str] = UNSET,
        account_id: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        opportunity_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        campaign_id: OptionalNullable[str] = UNSET,
        case_id: OptionalNullable[str] = UNSET,
        asset_id: OptionalNullable[str] = UNSET,
        contract_id: OptionalNullable[str] = UNSET,
        product_id: OptionalNullable[str] = UNSET,
        solution_id: OptionalNullable[str] = UNSET,
        custom_object_id: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        note: OptionalNullable[str] = UNSET,
        location: OptionalNullable[str] = UNSET,
        location_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        all_day_event: OptionalNullable[bool] = UNSET,
        private: OptionalNullable[bool] = UNSET,
        group_event: OptionalNullable[bool] = UNSET,
        event_sub_type: OptionalNullable[str] = UNSET,
        group_event_type: OptionalNullable[str] = UNSET,
        child: OptionalNullable[bool] = UNSET,
        archived: OptionalNullable[bool] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        show_as: OptionalNullable[models.ShowAs] = UNSET,
        done: OptionalNullable[bool] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        activity_date: OptionalNullable[str] = UNSET,
        end_date: OptionalNullable[str] = UNSET,
        recurrent: Optional[bool] = None,
        reminder_datetime: OptionalNullable[str] = UNSET,
        reminder_set: OptionalNullable[bool] = UNSET,
        video_conference_url: OptionalNullable[str] = UNSET,
        video_conference_id: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        attendees: Optional[
            Union[
                List[models.ActivityAttendeeInput],
                List[models.ActivityAttendeeInputTypedDict],
            ]
        ] = None,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesAddResponse:
        r"""Create activity

        Create activity

        :param type: The type of the activity
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param activity_datetime: The date and time of the activity
        :param duration_seconds: The duration of the activity in seconds
        :param user_id: The user related to the activity
        :param account_id: The account related to the activity
        :param contact_id: The contact related to the activity
        :param company_id: The company related to the activity
        :param opportunity_id: The opportunity related to the activity
        :param lead_id: The lead related to the activity
        :param owner_id: The owner of the activity
        :param campaign_id: The campaign related to the activity
        :param case_id: The case related to the activity
        :param asset_id: The asset related to the activity
        :param contract_id: The contract related to the activity
        :param product_id: The product related to the activity
        :param solution_id: The solution related to the activity
        :param custom_object_id: The custom object related to the activity
        :param title: The title of the activity
        :param description: A description of the activity
        :param note: An internal note about the activity
        :param location: The location of the activity
        :param location_address:
        :param all_day_event: Whether the Activity is an all day event or not
        :param private: Whether the Activity is private or not
        :param group_event: Whether the Activity is a group event or not
        :param event_sub_type: The sub type of the group event
        :param group_event_type: The type of the group event
        :param child: Whether the activity is a child of another activity or not
        :param archived: Whether the activity is archived or not
        :param deleted: Whether the activity is deleted or not
        :param show_as:
        :param done: Whether the Activity is done or not
        :param start_datetime: The start date and time of the activity
        :param end_datetime: The end date and time of the activity
        :param activity_date: The date of the activity
        :param end_date: The end date of the activity
        :param recurrent: Whether the activity is recurrent or not
        :param reminder_datetime: The date and time of the reminder
        :param reminder_set: Whether the reminder is set or not
        :param video_conference_url: The URL of the video conference
        :param video_conference_id: The ID of the video conference
        :param custom_fields: Custom fields of the activity
        :param attendees:
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            activity=models.ActivityInput(
                activity_datetime=activity_datetime,
                duration_seconds=duration_seconds,
                user_id=user_id,
                account_id=account_id,
                contact_id=contact_id,
                company_id=company_id,
                opportunity_id=opportunity_id,
                lead_id=lead_id,
                owner_id=owner_id,
                campaign_id=campaign_id,
                case_id=case_id,
                asset_id=asset_id,
                contract_id=contract_id,
                product_id=product_id,
                solution_id=solution_id,
                custom_object_id=custom_object_id,
                type=type_,
                title=title,
                description=description,
                note=note,
                location=location,
                location_address=utils.get_pydantic_model(
                    location_address, Optional[models.Address]
                ),
                all_day_event=all_day_event,
                private=private,
                group_event=group_event,
                event_sub_type=event_sub_type,
                group_event_type=group_event_type,
                child=child,
                archived=archived,
                deleted=deleted,
                show_as=show_as,
                done=done,
                start_datetime=start_datetime,
                end_datetime=end_datetime,
                activity_date=activity_date,
                end_date=end_date,
                recurrent=recurrent,
                reminder_datetime=reminder_datetime,
                reminder_set=reminder_set,
                video_conference_url=video_conference_url,
                video_conference_id=video_conference_id,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                attendees=utils.get_pydantic_model(
                    attendees, Optional[List[models.ActivityAttendeeInput]]
                ),
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/crm/activities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.activity, False, False, "json", models.ActivityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmActivitiesAddResponse(
                create_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesOneResponse:
        r"""Get activity

        Get activity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesOneResponse(
                get_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesOneResponse:
        r"""Get activity

        Get activity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesOneResponse(
                get_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        id: str,
        type_: Nullable[models.ActivityType],
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        activity_datetime: OptionalNullable[str] = UNSET,
        duration_seconds: OptionalNullable[int] = UNSET,
        user_id: OptionalNullable[str] = UNSET,
        account_id: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        opportunity_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        campaign_id: OptionalNullable[str] = UNSET,
        case_id: OptionalNullable[str] = UNSET,
        asset_id: OptionalNullable[str] = UNSET,
        contract_id: OptionalNullable[str] = UNSET,
        product_id: OptionalNullable[str] = UNSET,
        solution_id: OptionalNullable[str] = UNSET,
        custom_object_id: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        note: OptionalNullable[str] = UNSET,
        location: OptionalNullable[str] = UNSET,
        location_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        all_day_event: OptionalNullable[bool] = UNSET,
        private: OptionalNullable[bool] = UNSET,
        group_event: OptionalNullable[bool] = UNSET,
        event_sub_type: OptionalNullable[str] = UNSET,
        group_event_type: OptionalNullable[str] = UNSET,
        child: OptionalNullable[bool] = UNSET,
        archived: OptionalNullable[bool] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        show_as: OptionalNullable[models.ShowAs] = UNSET,
        done: OptionalNullable[bool] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        activity_date: OptionalNullable[str] = UNSET,
        end_date: OptionalNullable[str] = UNSET,
        recurrent: Optional[bool] = None,
        reminder_datetime: OptionalNullable[str] = UNSET,
        reminder_set: OptionalNullable[bool] = UNSET,
        video_conference_url: OptionalNullable[str] = UNSET,
        video_conference_id: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        attendees: Optional[
            Union[
                List[models.ActivityAttendeeInput],
                List[models.ActivityAttendeeInputTypedDict],
            ]
        ] = None,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesUpdateResponse:
        r"""Update activity

        Update activity

        :param id: ID of the record you are acting upon.
        :param type: The type of the activity
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param activity_datetime: The date and time of the activity
        :param duration_seconds: The duration of the activity in seconds
        :param user_id: The user related to the activity
        :param account_id: The account related to the activity
        :param contact_id: The contact related to the activity
        :param company_id: The company related to the activity
        :param opportunity_id: The opportunity related to the activity
        :param lead_id: The lead related to the activity
        :param owner_id: The owner of the activity
        :param campaign_id: The campaign related to the activity
        :param case_id: The case related to the activity
        :param asset_id: The asset related to the activity
        :param contract_id: The contract related to the activity
        :param product_id: The product related to the activity
        :param solution_id: The solution related to the activity
        :param custom_object_id: The custom object related to the activity
        :param title: The title of the activity
        :param description: A description of the activity
        :param note: An internal note about the activity
        :param location: The location of the activity
        :param location_address:
        :param all_day_event: Whether the Activity is an all day event or not
        :param private: Whether the Activity is private or not
        :param group_event: Whether the Activity is a group event or not
        :param event_sub_type: The sub type of the group event
        :param group_event_type: The type of the group event
        :param child: Whether the activity is a child of another activity or not
        :param archived: Whether the activity is archived or not
        :param deleted: Whether the activity is deleted or not
        :param show_as:
        :param done: Whether the Activity is done or not
        :param start_datetime: The start date and time of the activity
        :param end_datetime: The end date and time of the activity
        :param activity_date: The date of the activity
        :param end_date: The end date of the activity
        :param recurrent: Whether the activity is recurrent or not
        :param reminder_datetime: The date and time of the reminder
        :param reminder_set: Whether the reminder is set or not
        :param video_conference_url: The URL of the video conference
        :param video_conference_id: The ID of the video conference
        :param custom_fields: Custom fields of the activity
        :param attendees:
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            activity=models.ActivityInput(
                activity_datetime=activity_datetime,
                duration_seconds=duration_seconds,
                user_id=user_id,
                account_id=account_id,
                contact_id=contact_id,
                company_id=company_id,
                opportunity_id=opportunity_id,
                lead_id=lead_id,
                owner_id=owner_id,
                campaign_id=campaign_id,
                case_id=case_id,
                asset_id=asset_id,
                contract_id=contract_id,
                product_id=product_id,
                solution_id=solution_id,
                custom_object_id=custom_object_id,
                type=type_,
                title=title,
                description=description,
                note=note,
                location=location,
                location_address=utils.get_pydantic_model(
                    location_address, Optional[models.Address]
                ),
                all_day_event=all_day_event,
                private=private,
                group_event=group_event,
                event_sub_type=event_sub_type,
                group_event_type=group_event_type,
                child=child,
                archived=archived,
                deleted=deleted,
                show_as=show_as,
                done=done,
                start_datetime=start_datetime,
                end_datetime=end_datetime,
                activity_date=activity_date,
                end_date=end_date,
                recurrent=recurrent,
                reminder_datetime=reminder_datetime,
                reminder_set=reminder_set,
                video_conference_url=video_conference_url,
                video_conference_id=video_conference_id,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                attendees=utils.get_pydantic_model(
                    attendees, Optional[List[models.ActivityAttendeeInput]]
                ),
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.activity, False, False, "json", models.ActivityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesUpdateResponse(
                update_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        id: str,
        type_: Nullable[models.ActivityType],
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        activity_datetime: OptionalNullable[str] = UNSET,
        duration_seconds: OptionalNullable[int] = UNSET,
        user_id: OptionalNullable[str] = UNSET,
        account_id: OptionalNullable[str] = UNSET,
        contact_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        opportunity_id: OptionalNullable[str] = UNSET,
        lead_id: OptionalNullable[str] = UNSET,
        owner_id: OptionalNullable[str] = UNSET,
        campaign_id: OptionalNullable[str] = UNSET,
        case_id: OptionalNullable[str] = UNSET,
        asset_id: OptionalNullable[str] = UNSET,
        contract_id: OptionalNullable[str] = UNSET,
        product_id: OptionalNullable[str] = UNSET,
        solution_id: OptionalNullable[str] = UNSET,
        custom_object_id: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        note: OptionalNullable[str] = UNSET,
        location: OptionalNullable[str] = UNSET,
        location_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        all_day_event: OptionalNullable[bool] = UNSET,
        private: OptionalNullable[bool] = UNSET,
        group_event: OptionalNullable[bool] = UNSET,
        event_sub_type: OptionalNullable[str] = UNSET,
        group_event_type: OptionalNullable[str] = UNSET,
        child: OptionalNullable[bool] = UNSET,
        archived: OptionalNullable[bool] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        show_as: OptionalNullable[models.ShowAs] = UNSET,
        done: OptionalNullable[bool] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        activity_date: OptionalNullable[str] = UNSET,
        end_date: OptionalNullable[str] = UNSET,
        recurrent: Optional[bool] = None,
        reminder_datetime: OptionalNullable[str] = UNSET,
        reminder_set: OptionalNullable[bool] = UNSET,
        video_conference_url: OptionalNullable[str] = UNSET,
        video_conference_id: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        attendees: Optional[
            Union[
                List[models.ActivityAttendeeInput],
                List[models.ActivityAttendeeInputTypedDict],
            ]
        ] = None,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesUpdateResponse:
        r"""Update activity

        Update activity

        :param id: ID of the record you are acting upon.
        :param type: The type of the activity
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param activity_datetime: The date and time of the activity
        :param duration_seconds: The duration of the activity in seconds
        :param user_id: The user related to the activity
        :param account_id: The account related to the activity
        :param contact_id: The contact related to the activity
        :param company_id: The company related to the activity
        :param opportunity_id: The opportunity related to the activity
        :param lead_id: The lead related to the activity
        :param owner_id: The owner of the activity
        :param campaign_id: The campaign related to the activity
        :param case_id: The case related to the activity
        :param asset_id: The asset related to the activity
        :param contract_id: The contract related to the activity
        :param product_id: The product related to the activity
        :param solution_id: The solution related to the activity
        :param custom_object_id: The custom object related to the activity
        :param title: The title of the activity
        :param description: A description of the activity
        :param note: An internal note about the activity
        :param location: The location of the activity
        :param location_address:
        :param all_day_event: Whether the Activity is an all day event or not
        :param private: Whether the Activity is private or not
        :param group_event: Whether the Activity is a group event or not
        :param event_sub_type: The sub type of the group event
        :param group_event_type: The type of the group event
        :param child: Whether the activity is a child of another activity or not
        :param archived: Whether the activity is archived or not
        :param deleted: Whether the activity is deleted or not
        :param show_as:
        :param done: Whether the Activity is done or not
        :param start_datetime: The start date and time of the activity
        :param end_datetime: The end date and time of the activity
        :param activity_date: The date of the activity
        :param end_date: The end date of the activity
        :param recurrent: Whether the activity is recurrent or not
        :param reminder_datetime: The date and time of the reminder
        :param reminder_set: Whether the reminder is set or not
        :param video_conference_url: The URL of the video conference
        :param video_conference_id: The ID of the video conference
        :param custom_fields: Custom fields of the activity
        :param attendees:
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            activity=models.ActivityInput(
                activity_datetime=activity_datetime,
                duration_seconds=duration_seconds,
                user_id=user_id,
                account_id=account_id,
                contact_id=contact_id,
                company_id=company_id,
                opportunity_id=opportunity_id,
                lead_id=lead_id,
                owner_id=owner_id,
                campaign_id=campaign_id,
                case_id=case_id,
                asset_id=asset_id,
                contract_id=contract_id,
                product_id=product_id,
                solution_id=solution_id,
                custom_object_id=custom_object_id,
                type=type_,
                title=title,
                description=description,
                note=note,
                location=location,
                location_address=utils.get_pydantic_model(
                    location_address, Optional[models.Address]
                ),
                all_day_event=all_day_event,
                private=private,
                group_event=group_event,
                event_sub_type=event_sub_type,
                group_event_type=group_event_type,
                child=child,
                archived=archived,
                deleted=deleted,
                show_as=show_as,
                done=done,
                start_datetime=start_datetime,
                end_datetime=end_datetime,
                activity_date=activity_date,
                end_date=end_date,
                recurrent=recurrent,
                reminder_datetime=reminder_datetime,
                reminder_set=reminder_set,
                video_conference_url=video_conference_url,
                video_conference_id=video_conference_id,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                attendees=utils.get_pydantic_model(
                    attendees, Optional[List[models.ActivityAttendeeInput]]
                ),
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.activity, False, False, "json", models.ActivityInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesUpdateResponse(
                update_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesDeleteResponse:
        r"""Delete activity

        Delete activity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="DELETE",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesDeleteResponse(
                delete_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmActivitiesDeleteResponse:
        r"""Delete activity

        Delete activity

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmActivitiesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/crm/activities/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmActivitiesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="crm.activitiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmActivitiesDeleteResponse(
                delete_activity_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteActivityResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.BadRequestResponseData
            )
            raise models.BadRequestResponse(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnauthorizedResponseData
            )
            raise models.UnauthorizedResponse(data=response_data)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.NotFoundResponseData
            )
            raise models.NotFoundResponse(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableResponseData
            )
            raise models.UnprocessableResponse(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmActivitiesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .website import Website, WebsiteTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class HrisCompanyStatus(str, Enum):
    ACTIVE = "active"
    INACTIVE = "inactive"
    TRIAL = "trial"
    OTHER = "other"


class HrisCompanyTypedDict(TypedDict):
    legal_name: Nullable[str]
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    display_name: NotRequired[Nullable[str]]
    subdomain: NotRequired[Nullable[str]]
    status: NotRequired[HrisCompanyStatus]
    company_number: NotRequired[Nullable[str]]
    r"""An Company Number, Company ID or Company Code, is a unique number that has been assigned to each company."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    debtor_id: NotRequired[Nullable[str]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    deleted: NotRequired[bool]
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class HrisCompany(BaseModel):
    legal_name: Nullable[str]

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    display_name: OptionalNullable[str] = UNSET

    subdomain: OptionalNullable[str] = UNSET

    status: Optional[HrisCompanyStatus] = None

    company_number: OptionalNullable[str] = UNSET
    r"""An Company Number, Company ID or Company Code, is a unique number that has been assigned to each company."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    websites: Optional[List[Website]] = None

    debtor_id: OptionalNullable[str] = UNSET

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    deleted: Optional[bool] = None

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "display_name",
            "subdomain",
            "status",
            "company_number",
            "currency",
            "addresses",
            "phone_numbers",
            "emails",
            "websites",
            "debtor_id",
            "custom_mappings",
            "deleted",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "legal_name",
            "display_name",
            "subdomain",
            "company_number",
            "currency",
            "debtor_id",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class HrisCompanyInputTypedDict(TypedDict):
    legal_name: Nullable[str]
    display_name: NotRequired[Nullable[str]]
    subdomain: NotRequired[Nullable[str]]
    status: NotRequired[HrisCompanyStatus]
    company_number: NotRequired[Nullable[str]]
    r"""An Company Number, Company ID or Company Code, is a unique number that has been assigned to each company."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    debtor_id: NotRequired[Nullable[str]]
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class HrisCompanyInput(BaseModel):
    legal_name: Nullable[str]

    display_name: OptionalNullable[str] = UNSET

    subdomain: OptionalNullable[str] = UNSET

    status: Optional[HrisCompanyStatus] = None

    company_number: OptionalNullable[str] = UNSET
    r"""An Company Number, Company ID or Company Code, is a unique number that has been assigned to each company."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    websites: Optional[List[Website]] = None

    debtor_id: OptionalNullable[str] = UNSET

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "display_name",
            "subdomain",
            "status",
            "company_number",
            "currency",
            "addresses",
            "phone_numbers",
            "emails",
            "websites",
            "debtor_id",
            "pass_through",
        ]
        nullable_fields = [
            "legal_name",
            "display_name",
            "subdomain",
            "company_number",
            "currency",
            "debtor_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class PhoneNumberType(str, Enum):
    r"""The type of phone number"""

    PRIMARY = "primary"
    SECONDARY = "secondary"
    HOME = "home"
    WORK = "work"
    OFFICE = "office"
    MOBILE = "mobile"
    ASSISTANT = "assistant"
    FAX = "fax"
    DIRECT_DIAL_IN = "direct-dial-in"
    PERSONAL = "personal"
    OTHER = "other"


class PhoneNumberTypedDict(TypedDict):
    number: str
    r"""The phone number"""
    id: NotRequired[Nullable[str]]
    r"""Unique identifier of the phone number"""
    country_code: NotRequired[Nullable[str]]
    r"""The country code of the phone number, e.g. +1"""
    area_code: NotRequired[Nullable[str]]
    r"""The area code of the phone number, e.g. 323"""
    extension: NotRequired[Nullable[str]]
    r"""The extension of the phone number"""
    type: NotRequired[Nullable[PhoneNumberType]]
    r"""The type of phone number"""


class PhoneNumber(BaseModel):
    number: str
    r"""The phone number"""

    id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the phone number"""

    country_code: OptionalNullable[str] = UNSET
    r"""The country code of the phone number, e.g. +1"""

    area_code: OptionalNullable[str] = UNSET
    r"""The area code of the phone number, e.g. 323"""

    extension: OptionalNullable[str] = UNSET
    r"""The extension of the phone number"""

    type: OptionalNullable[PhoneNumberType] = UNSET
    r"""The type of phone number"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "country_code", "area_code", "extension", "type"]
        nullable_fields = ["id", "country_code", "area_code", "extension", "type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class UpdateFileRequestTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the file."""
    description: NotRequired[str]
    r"""Optional description of the file."""
    parent_folder_id: NotRequired[str]
    r"""The parent folder to create the new file within. This can be an ID or a path depending on the downstream folder. Please see the connector section below to see downstream specific gotchas."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class UpdateFileRequest(BaseModel):
    name: Optional[str] = None
    r"""The name of the file."""

    description: Optional[str] = None
    r"""Optional description of the file."""

    parent_folder_id: Optional[str] = None
    r"""The parent folder to create the new file within. This can be an ID or a path depending on the downstream folder. Please see the connector section below to see downstream specific gotchas."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourcestatus import ResourceStatus
from apideck_unify.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LinkedResourcesTypedDict(TypedDict):
    id: NotRequired[str]
    r"""ID of the resource, typically a lowercased version of name."""
    unified_property: NotRequired[str]
    r"""Name of the property in our Unified API."""


class LinkedResources(BaseModel):
    id: Optional[str] = None
    r"""ID of the resource, typically a lowercased version of name."""

    unified_property: Optional[str] = None
    r"""Name of the property in our Unified API."""


class SchemaTypedDict(TypedDict):
    r"""JSON Schema of the resource in our Unified API"""


class Schema(BaseModel):
    r"""JSON Schema of the resource in our Unified API"""


class APIResourceTypedDict(TypedDict):
    id: NotRequired[str]
    r"""ID of the resource, typically a lowercased version of name."""
    name: NotRequired[str]
    r"""Name of the resource (plural)"""
    status: NotRequired[ResourceStatus]
    r"""Status of the resource. Resources with status live or beta are callable."""
    linked_resources: NotRequired[List[LinkedResourcesTypedDict]]
    r"""List of linked resources."""
    schema_: NotRequired[SchemaTypedDict]
    r"""JSON Schema of the resource in our Unified API"""


class APIResource(BaseModel):
    id: Optional[str] = None
    r"""ID of the resource, typically a lowercased version of name."""

    name: Optional[str] = None
    r"""Name of the resource (plural)"""

    status: Optional[ResourceStatus] = None
    r"""Status of the resource. Resources with status live or beta are callable."""

    linked_resources: Optional[List[LinkedResources]] = None
    r"""List of linked resources."""

    schema_: Annotated[Optional[Schema], pydantic.Field(alias="schema")] = None
    r"""JSON Schema of the resource in our Unified API"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class CustomMappingsTypedDict(TypedDict):
    r"""When custom mappings are configured on the resource, the result is included here."""


class CustomMappings(BaseModel):
    r"""When custom mappings are configured on the resource, the result is included here."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccount import BankAccount, BankAccountTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Classification(str, Enum):
    r"""The classification of account."""

    ASSET = "asset"
    EQUITY = "equity"
    EXPENSE = "expense"
    LIABILITY = "liability"
    REVENUE = "revenue"
    INCOME = "income"
    OTHER_INCOME = "other_income"
    OTHER_EXPENSE = "other_expense"
    COSTS_OF_SALES = "costs_of_sales"
    OTHER = "other"


class LedgerAccountType(str, Enum):
    r"""The type of account."""

    ACCOUNTS_RECEIVABLE = "accounts_receivable"
    REVENUE = "revenue"
    SALES = "sales"
    OTHER_INCOME = "other_income"
    BANK = "bank"
    CURRENT_ASSET = "current_asset"
    FIXED_ASSET = "fixed_asset"
    NON_CURRENT_ASSET = "non_current_asset"
    OTHER_ASSET = "other_asset"
    BALANCESHEET = "balancesheet"
    EQUITY = "equity"
    EXPENSE = "expense"
    OTHER_EXPENSE = "other_expense"
    COSTS_OF_SALES = "costs_of_sales"
    ACCOUNTS_PAYABLE = "accounts_payable"
    CREDIT_CARD = "credit_card"
    CURRENT_LIABILITY = "current_liability"
    NON_CURRENT_LIABILITY = "non_current_liability"
    OTHER_LIABILITY = "other_liability"
    OTHER = "other"


class AccountStatus(str, Enum):
    r"""The status of the account."""

    ACTIVE = "active"
    INACTIVE = "inactive"
    ARCHIVED = "archived"


class CategoriesTypedDict(TypedDict):
    id: NotRequired[str]
    name: NotRequired[str]
    r"""The name of the category."""


class Categories(BaseModel):
    id: Optional[str] = None

    name: Optional[str] = None
    r"""The name of the category."""


class ParentAccountTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The ID of the parent account."""
    name: NotRequired[str]
    r"""The name of the parent account."""
    display_id: NotRequired[str]
    r"""The human readable display ID used when displaying the parent account"""


class ParentAccount(BaseModel):
    id: Optional[str] = None
    r"""The ID of the parent account."""

    name: Optional[str] = None
    r"""The name of the parent account."""

    display_id: Optional[str] = None
    r"""The human readable display ID used when displaying the parent account"""


class SubAccountsTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The ID of the sub account."""
    account_sub_name: NotRequired[str]
    r"""The name of the sub account."""


class SubAccounts(BaseModel):
    id: Optional[str] = None
    r"""The ID of the sub account."""

    account_sub_name: Optional[str] = None
    r"""The name of the sub account."""


class LedgerAccountSubsidiariesTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The ID of the subsidiary."""


class LedgerAccountSubsidiaries(BaseModel):
    id: Optional[str] = None
    r"""The ID of the subsidiary."""


class LedgerAccountTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    display_id: NotRequired[str]
    r"""The human readable display ID used when displaying the account"""
    nominal_code: NotRequired[Nullable[str]]
    r"""The nominal code of the ledger account."""
    code: NotRequired[Nullable[str]]
    r"""The code assigned to the account."""
    classification: NotRequired[Nullable[Classification]]
    r"""The classification of account."""
    type: NotRequired[LedgerAccountType]
    r"""The type of account."""
    sub_type: NotRequired[Nullable[str]]
    r"""The sub type of account."""
    name: NotRequired[Nullable[str]]
    r"""The name of the account."""
    fully_qualified_name: NotRequired[Nullable[str]]
    r"""The fully qualified name of the account."""
    description: NotRequired[Nullable[str]]
    r"""The description of the account."""
    opening_balance: NotRequired[Nullable[float]]
    r"""The opening balance of the account."""
    current_balance: NotRequired[Nullable[float]]
    r"""The current balance of the account."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    tax_type: NotRequired[Nullable[str]]
    r"""The tax type of the account."""
    tax_rate: NotRequired[LinkedTaxRateTypedDict]
    level: NotRequired[Nullable[float]]
    active: NotRequired[Nullable[bool]]
    r"""Whether the account is active or not."""
    status: NotRequired[Nullable[AccountStatus]]
    r"""The status of the account."""
    header: NotRequired[Nullable[bool]]
    r"""Whether the account is a header or not."""
    bank_account: NotRequired[BankAccountTypedDict]
    categories: NotRequired[List[CategoriesTypedDict]]
    r"""The categories of the account."""
    parent_account: NotRequired[ParentAccountTypedDict]
    sub_account: NotRequired[Nullable[bool]]
    r"""Whether the account is a sub account or not."""
    sub_accounts: NotRequired[List[SubAccountsTypedDict]]
    r"""The sub accounts of the account."""
    last_reconciliation_date: NotRequired[Nullable[date]]
    r"""Reconciliation Date means the last calendar day of each Reconciliation Period."""
    subsidiaries: NotRequired[List[LedgerAccountSubsidiariesTypedDict]]
    r"""The subsidiaries the account belongs to."""
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class LedgerAccount(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    display_id: Optional[str] = None
    r"""The human readable display ID used when displaying the account"""

    nominal_code: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The nominal code of the ledger account."""

    code: OptionalNullable[str] = UNSET
    r"""The code assigned to the account."""

    classification: OptionalNullable[Classification] = UNSET
    r"""The classification of account."""

    type: Optional[LedgerAccountType] = None
    r"""The type of account."""

    sub_type: OptionalNullable[str] = UNSET
    r"""The sub type of account."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the account."""

    fully_qualified_name: OptionalNullable[str] = UNSET
    r"""The fully qualified name of the account."""

    description: OptionalNullable[str] = UNSET
    r"""The description of the account."""

    opening_balance: OptionalNullable[float] = UNSET
    r"""The opening balance of the account."""

    current_balance: OptionalNullable[float] = UNSET
    r"""The current balance of the account."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    tax_type: OptionalNullable[str] = UNSET
    r"""The tax type of the account."""

    tax_rate: Optional[LinkedTaxRate] = None

    level: OptionalNullable[float] = UNSET

    active: OptionalNullable[bool] = UNSET
    r"""Whether the account is active or not."""

    status: OptionalNullable[AccountStatus] = UNSET
    r"""The status of the account."""

    header: OptionalNullable[bool] = UNSET
    r"""Whether the account is a header or not."""

    bank_account: Optional[BankAccount] = None

    categories: Optional[List[Categories]] = None
    r"""The categories of the account."""

    parent_account: Optional[ParentAccount] = None

    sub_account: OptionalNullable[bool] = UNSET
    r"""Whether the account is a sub account or not."""

    sub_accounts: Optional[List[SubAccounts]] = None
    r"""The sub accounts of the account."""

    last_reconciliation_date: OptionalNullable[date] = UNSET
    r"""Reconciliation Date means the last calendar day of each Reconciliation Period."""

    subsidiaries: Optional[List[LedgerAccountSubsidiaries]] = None
    r"""The subsidiaries the account belongs to."""

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "display_id",
            "nominal_code",
            "code",
            "classification",
            "type",
            "sub_type",
            "name",
            "fully_qualified_name",
            "description",
            "opening_balance",
            "current_balance",
            "currency",
            "tax_type",
            "tax_rate",
            "level",
            "active",
            "status",
            "header",
            "bank_account",
            "categories",
            "parent_account",
            "sub_account",
            "sub_accounts",
            "last_reconciliation_date",
            "subsidiaries",
            "custom_mappings",
            "custom_fields",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "nominal_code",
            "code",
            "classification",
            "sub_type",
            "name",
            "fully_qualified_name",
            "description",
            "opening_balance",
            "current_balance",
            "currency",
            "tax_type",
            "level",
            "active",
            "status",
            "header",
            "sub_account",
            "last_reconciliation_date",
            "custom_mappings",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class LedgerAccountInputTypedDict(TypedDict):
    display_id: NotRequired[str]
    r"""The human readable display ID used when displaying the account"""
    nominal_code: NotRequired[Nullable[str]]
    r"""The nominal code of the ledger account."""
    code: NotRequired[Nullable[str]]
    r"""The code assigned to the account."""
    classification: NotRequired[Nullable[Classification]]
    r"""The classification of account."""
    type: NotRequired[LedgerAccountType]
    r"""The type of account."""
    sub_type: NotRequired[Nullable[str]]
    r"""The sub type of account."""
    name: NotRequired[Nullable[str]]
    r"""The name of the account."""
    fully_qualified_name: NotRequired[Nullable[str]]
    r"""The fully qualified name of the account."""
    description: NotRequired[Nullable[str]]
    r"""The description of the account."""
    opening_balance: NotRequired[Nullable[float]]
    r"""The opening balance of the account."""
    current_balance: NotRequired[Nullable[float]]
    r"""The current balance of the account."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    tax_type: NotRequired[Nullable[str]]
    r"""The tax type of the account."""
    tax_rate: NotRequired[LinkedTaxRateInputTypedDict]
    level: NotRequired[Nullable[float]]
    active: NotRequired[Nullable[bool]]
    r"""Whether the account is active or not."""
    status: NotRequired[Nullable[AccountStatus]]
    r"""The status of the account."""
    header: NotRequired[Nullable[bool]]
    r"""Whether the account is a header or not."""
    bank_account: NotRequired[BankAccountTypedDict]
    parent_account: NotRequired[ParentAccountTypedDict]
    sub_account: NotRequired[Nullable[bool]]
    r"""Whether the account is a sub account or not."""
    last_reconciliation_date: NotRequired[Nullable[date]]
    r"""Reconciliation Date means the last calendar day of each Reconciliation Period."""
    subsidiaries: NotRequired[List[LedgerAccountSubsidiariesTypedDict]]
    r"""The subsidiaries the account belongs to."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class LedgerAccountInput(BaseModel):
    display_id: Optional[str] = None
    r"""The human readable display ID used when displaying the account"""

    nominal_code: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The nominal code of the ledger account."""

    code: OptionalNullable[str] = UNSET
    r"""The code assigned to the account."""

    classification: OptionalNullable[Classification] = UNSET
    r"""The classification of account."""

    type: Optional[LedgerAccountType] = None
    r"""The type of account."""

    sub_type: OptionalNullable[str] = UNSET
    r"""The sub type of account."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the account."""

    fully_qualified_name: OptionalNullable[str] = UNSET
    r"""The fully qualified name of the account."""

    description: OptionalNullable[str] = UNSET
    r"""The description of the account."""

    opening_balance: OptionalNullable[float] = UNSET
    r"""The opening balance of the account."""

    current_balance: OptionalNullable[float] = UNSET
    r"""The current balance of the account."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    tax_type: OptionalNullable[str] = UNSET
    r"""The tax type of the account."""

    tax_rate: Optional[LinkedTaxRateInput] = None

    level: OptionalNullable[float] = UNSET

    active: OptionalNullable[bool] = UNSET
    r"""Whether the account is active or not."""

    status: OptionalNullable[AccountStatus] = UNSET
    r"""The status of the account."""

    header: OptionalNullable[bool] = UNSET
    r"""Whether the account is a header or not."""

    bank_account: Optional[BankAccount] = None

    parent_account: Optional[ParentAccount] = None

    sub_account: OptionalNullable[bool] = UNSET
    r"""Whether the account is a sub account or not."""

    last_reconciliation_date: OptionalNullable[date] = UNSET
    r"""Reconciliation Date means the last calendar day of each Reconciliation Period."""

    subsidiaries: Optional[List[LedgerAccountSubsidiaries]] = None
    r"""The subsidiaries the account belongs to."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "display_id",
            "nominal_code",
            "code",
            "classification",
            "type",
            "sub_type",
            "name",
            "fully_qualified_name",
            "description",
            "opening_balance",
            "current_balance",
            "currency",
            "tax_type",
            "tax_rate",
            "level",
            "active",
            "status",
            "header",
            "bank_account",
            "parent_account",
            "sub_account",
            "last_reconciliation_date",
            "subsidiaries",
            "custom_fields",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "nominal_code",
            "code",
            "classification",
            "sub_type",
            "name",
            "fully_qualified_name",
            "description",
            "opening_balance",
            "current_balance",
            "currency",
            "tax_type",
            "level",
            "active",
            "status",
            "header",
            "sub_account",
            "last_reconciliation_date",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class CredentialsTypedDict(TypedDict):
    refresh_token: NotRequired[Nullable[str]]
    r"""The refresh token can be used to obtain a new access token."""
    access_token: NotRequired[str]
    r"""Access token"""
    issued_at: NotRequired[Nullable[datetime]]
    r"""The datetime at which the token was issued. If omitted the token will be queued for refresh."""
    expires_in: NotRequired[Nullable[int]]
    r"""The number of seconds until the token expires. If omitted the token will be queued for refresh."""


class Credentials(BaseModel):
    refresh_token: OptionalNullable[str] = UNSET
    r"""The refresh token can be used to obtain a new access token."""

    access_token: Optional[str] = None
    r"""Access token"""

    issued_at: OptionalNullable[datetime] = UNSET
    r"""The datetime at which the token was issued. If omitted the token will be queued for refresh."""

    expires_in: OptionalNullable[int] = UNSET
    r"""The number of seconds until the token expires. If omitted the token will be queued for refresh."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["refresh_token", "access_token", "issued_at", "expires_in"]
        nullable_fields = ["refresh_token", "issued_at", "expires_in"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SettingsTypedDict(TypedDict):
    r"""Connection settings. Values will persist to `form_fields` with corresponding id"""


class Settings(BaseModel):
    r"""Connection settings. Values will persist to `form_fields` with corresponding id"""


class ConnectionImportDataTypedDict(TypedDict):
    credentials: NotRequired[CredentialsTypedDict]
    settings: NotRequired[Nullable[SettingsTypedDict]]
    r"""Connection settings. Values will persist to `form_fields` with corresponding id"""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Attach your own consumer specific metadata"""


class ConnectionImportData(BaseModel):
    credentials: Optional[Credentials] = None

    settings: OptionalNullable[Settings] = UNSET
    r"""Connection settings. Values will persist to `form_fields` with corresponding id"""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Attach your own consumer specific metadata"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["credentials", "settings", "metadata"]
        nullable_fields = ["settings", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .outstandingbalancebycurrency import (
    OutstandingBalanceByCurrency,
    OutstandingBalanceByCurrencyTypedDict,
)
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class OutstandingBalanceBySupplierTypedDict(TypedDict):
    supplier_id: NotRequired[str]
    r"""Unique identifier for the supplier."""
    supplier_name: NotRequired[str]
    r"""Full name of the supplier."""
    outstanding_balances_by_currency: NotRequired[
        List[OutstandingBalanceByCurrencyTypedDict]
    ]


class OutstandingBalanceBySupplier(BaseModel):
    supplier_id: Optional[str] = None
    r"""Unique identifier for the supplier."""

    supplier_name: Optional[str] = None
    r"""Full name of the supplier."""

    outstanding_balances_by_currency: Optional[List[OutstandingBalanceByCurrency]] = (
        None
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .outstandingbalancebycurrency import (
    OutstandingBalanceByCurrency,
    OutstandingBalanceByCurrencyTypedDict,
)
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class OutstandingBalanceByCustomerTypedDict(TypedDict):
    customer_id: NotRequired[str]
    r"""Unique identifier for the customer."""
    customer_name: NotRequired[str]
    r"""Full name of the customer."""
    outstanding_balances_by_currency: NotRequired[
        List[OutstandingBalanceByCurrencyTypedDict]
    ]


class OutstandingBalanceByCustomer(BaseModel):
    customer_id: Optional[str] = None
    r"""Unique identifier for the customer."""

    customer_name: Optional[str] = None
    r"""Full name of the customer."""

    outstanding_balances_by_currency: Optional[List[OutstandingBalanceByCurrency]] = (
        None
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apideck_companies import ApideckCompanies
from apideck_unify.apideck_departments import ApideckDepartments
from apideck_unify.employeepayrolls import EmployeePayrolls
from apideck_unify.employees import Employees
from apideck_unify.employeeschedules_sdk import EmployeeSchedulesSDK
from apideck_unify.payrolls import Payrolls
from apideck_unify.timeoffrequests import TimeOffRequests


class Hris(BaseSDK):
    employees: Employees
    companies: ApideckCompanies
    departments: ApideckDepartments
    payrolls: Payrolls
    employee_payrolls: EmployeePayrolls
    employee_schedules: EmployeeSchedulesSDK
    time_off_requests: TimeOffRequests

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.employees = Employees(self.sdk_configuration)
        self.companies = ApideckCompanies(self.sdk_configuration)
        self.departments = ApideckDepartments(self.sdk_configuration)
        self.payrolls = Payrolls(self.sdk_configuration)
        self.employee_payrolls = EmployeePayrolls(self.sdk_configuration)
        self.employee_schedules = EmployeeSchedulesSDK(self.sdk_configuration)
        self.time_off_requests = TimeOffRequests(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ecommercediscount import EcommerceDiscount, EcommerceDiscountTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class OptionsTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for the option."""
    name: NotRequired[Nullable[str]]
    r"""The name of the option."""
    value: NotRequired[Nullable[str]]
    r"""The value of the option."""


class Options(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the option."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the option."""

    value: OptionalNullable[str] = UNSET
    r"""The value of the option."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "name", "value"]
        nullable_fields = ["id", "name", "value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceOrderLineItemTypedDict(TypedDict):
    r"""A single line item of an ecommerce order, representing a product or variant with associated options, quantity, and pricing information."""

    name: Nullable[str]
    r"""The name of the product or variant associated with the line item."""
    quantity: Nullable[str]
    r"""The quantity of the product or variant associated with the line item."""
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    product_id: NotRequired[Nullable[str]]
    r"""A unique identifier for the product associated with the line item."""
    variant_id: NotRequired[Nullable[str]]
    r"""A unique identifier for the variant of the product associated with the line item, if applicable."""
    sku: NotRequired[Nullable[str]]
    r"""The SKU of the product or variant associated with the line item."""
    description: NotRequired[Nullable[str]]
    r"""The description of the product or variant associated with the line item."""
    options: NotRequired[List[OptionsTypedDict]]
    unit_price: NotRequired[Nullable[str]]
    r"""The unit price of the product or variant associated with the line item."""
    tax_rate: NotRequired[Nullable[str]]
    r"""The tax rate applied to the product or variant associated with the line item."""
    tax_amount: NotRequired[Nullable[str]]
    r"""The total tax amount applied to the product or variant associated with the line item."""
    is_refunded: NotRequired[Nullable[bool]]
    r"""Whether the line item has been refunded."""
    refunded_amount: NotRequired[Nullable[str]]
    r"""The amount of the line item that has been refunded."""
    refunded_quantity: NotRequired[Nullable[str]]
    r"""The quantity of the line item that has been refunded."""
    sub_total: NotRequired[Nullable[str]]
    r"""The sub total for the product(s) or variant associated with the line item, excluding taxes and discounts."""
    total_amount: NotRequired[Nullable[str]]
    r"""The total amount for the product(s) or variant associated with the line item, including taxes and discounts."""
    discounts: NotRequired[List[EcommerceDiscountTypedDict]]


class EcommerceOrderLineItem(BaseModel):
    r"""A single line item of an ecommerce order, representing a product or variant with associated options, quantity, and pricing information."""

    name: Nullable[str]
    r"""The name of the product or variant associated with the line item."""

    quantity: Nullable[str]
    r"""The quantity of the product or variant associated with the line item."""

    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    product_id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the product associated with the line item."""

    variant_id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the variant of the product associated with the line item, if applicable."""

    sku: OptionalNullable[str] = UNSET
    r"""The SKU of the product or variant associated with the line item."""

    description: OptionalNullable[str] = UNSET
    r"""The description of the product or variant associated with the line item."""

    options: Optional[List[Options]] = None

    unit_price: OptionalNullable[str] = UNSET
    r"""The unit price of the product or variant associated with the line item."""

    tax_rate: OptionalNullable[str] = UNSET
    r"""The tax rate applied to the product or variant associated with the line item."""

    tax_amount: OptionalNullable[str] = UNSET
    r"""The total tax amount applied to the product or variant associated with the line item."""

    is_refunded: OptionalNullable[bool] = UNSET
    r"""Whether the line item has been refunded."""

    refunded_amount: OptionalNullable[str] = UNSET
    r"""The amount of the line item that has been refunded."""

    refunded_quantity: OptionalNullable[str] = UNSET
    r"""The quantity of the line item that has been refunded."""

    sub_total: OptionalNullable[str] = UNSET
    r"""The sub total for the product(s) or variant associated with the line item, excluding taxes and discounts."""

    total_amount: OptionalNullable[str] = UNSET
    r"""The total amount for the product(s) or variant associated with the line item, including taxes and discounts."""

    discounts: Optional[List[EcommerceDiscount]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "product_id",
            "variant_id",
            "sku",
            "description",
            "options",
            "unit_price",
            "tax_rate",
            "tax_amount",
            "is_refunded",
            "refunded_amount",
            "refunded_quantity",
            "sub_total",
            "total_amount",
            "discounts",
        ]
        nullable_fields = [
            "id",
            "product_id",
            "variant_id",
            "sku",
            "name",
            "description",
            "quantity",
            "unit_price",
            "tax_rate",
            "tax_amount",
            "is_refunded",
            "refunded_amount",
            "refunded_quantity",
            "sub_total",
            "total_amount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
